#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from enum import Enum

class DataOutputFormats(Enum):
    JSON = "json"
    YAML = "yaml"